# Generated by Django 2.0.3 on 2018-03-26 00:00

from django.db import migrations, models
import ndh.models


class Migration(migrations.Migration):

    dependencies = [
        ('testapp', '0002_testmodellist'),
    ]

    operations = [
        migrations.CreateModel(
            name='TestModelPK',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
            bases=(ndh.models.Links, models.Model),
        ),
    ]



from typing import Dict

from django import template
from django.db.models.query import QuerySet
from django.urls import reverse
from django.utils.safestring import mark_safe

register = template.Library()


@register.simple_tag(takes_context=True)
def show_email(context: Dict, mail: str) -> str:
    request = context['request']
    if request.user.is_authenticated:
        content = f'<a href="mailto:{mail}">{mail}</a>'
    else:
        at, dot = (f'<span class="{tag}"></span>' for tag in ('at', 'dot'))
        content = mail.replace('@', at).replace('.', dot)
    return mark_safe(f'<span class="mail">{content}</span>')


@register.filter
def admin_url(obj) -> str:
    if isinstance(obj, QuerySet):
        obj = obj[0]._meta
        return reverse(f'admin:{obj.app_label}_{obj.model_name}_changelist')
    return reverse(f'admin:{obj._meta.app_label}_{obj._meta.model_name}_change', args=[obj.pk])


@register.simple_tag(takes_context=True)
def navbar_item(context, view_name: str, link: str) -> str:
    url = reverse(view_name)
    active = 'active' if url == context.request.path else ''
    return mark_safe(f'<li class="nav-item {active}"><a class="nav-link" href="{url}">{link}</a></li>')



"""Define URLs for the test app."""
from django.urls import path
from django.views.generic import DetailView, ListView, TemplateView

from . import views
from .models import TestModel, TestModelList, TestModelPK

CTX = {'email': 'test@example.org'}

app_name = 'testapp'
urlpatterns = [
    path('', ListView.as_view(model=TestModel), name='testmodels'),
    path('create', views.TestCreateView.as_view(), name='testmodel-add'),
    path('list', ListView.as_view(model=TestModelList), name='testmodellists'),
    path('pk/<int:pk>', DetailView.as_view(model=TestModelPK), name='testmodelpk'),
    path('settings', TemplateView.as_view(template_name='testapp/settings.html'), name='settings'),
    path('<slug:slug>', DetailView.as_view(model=TestModel), name='testmodel'),
    path('<slug:slug>/delete', views.TestDeleteView.as_view(), name='testmodel-del'),
]



# Generated by Django 2.0 on 2017-12-06 02:35

from typing import List, Tuple

from django.db import migrations, models

import autoslug.fields  # type:ignore
import ndh.models


class Migration(migrations.Migration):

    initial = True

    dependencies: List[Tuple[str, str]] = []

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=200, unique=True)),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from='name', unique=True)),
                ('year_in_school',
                 models.IntegerField(choices=[(1, 'freshman'), (2, 'sophomore'), (3, 'junior'), (4, 'senior')],
                                     default=1)),
                ('tests', models.IntegerField(default=42)),
                ('moment', models.DateTimeField()),
            ],
            options={
                'abstract': False,
            },
            bases=(ndh.models.Links, models.Model),
        ),
    ]



"""Main URLS for the test project."""
from django.contrib import admin
from django.urls import include, path
from django.views.generic import TemplateView

CTX = {'email': 'test@example.org'}

urlpatterns = [
    path(r'admin/', admin.site.urls),
    path('accounts/', include('django.contrib.auth.urls')),
    path('', TemplateView.as_view(template_name='base.html', extra_context=CTX), name='test'),
    path('test/', include('testapp.urls')),
]



"""Forms for the testapp."""
from django.forms import ModelForm

from ndh.forms import AccessibleDateTimeField

from .models import TestModel


class TestForm(ModelForm):
    """Form to test ndh.forms."""

    moment = AccessibleDateTimeField()

    class Meta:
        """ModelForm definitions."""

        model = TestModel
        fields = ('name', 'year_in_school', 'tests', 'moment')



"""Views for testapp."""
from django.urls import reverse_lazy
from django.views.generic import CreateView, DeleteView

from ndh.mixins import NDHDeleteMixin, NDHFormMixin, SuperUserRequiredMixin

from .forms import TestForm
from .models import TestModel


class TestCreateView(NDHFormMixin, CreateView):
    """View to test NDHFormMixin."""
    model = TestModel
    form_class = TestForm
    title = 'Create Test'


class TestDeleteView(NDHDeleteMixin, SuperUserRequiredMixin, DeleteView):
    """View to test other mixins."""
    model = TestModel
    success_url = reverse_lazy('testapp:testmodels')
    title = 'Delete Test'



"""Main entrypoint for test project & app."""
import os
from datetime import timedelta

from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse
from django.utils import timezone

from ndh.utils import get_env, query_sum

from .models import TestModel, TestModelList, TestModelPK


class TestNDH(TestCase):
    """Main class to test ndh functionnalities."""
    def test_models(self):
        """Test ndh.models."""
        # test creation
        self.assertEqual(TestModel.objects.count(), 0)
        TestModel.objects.create(name='Pipo 22 é@ü', moment=timezone.now())
        self.assertEqual(TestModel.objects.count(), 1)

        # test name & slug
        instance = TestModel.objects.first()
        self.assertEqual(instance.name, str(instance))
        self.assertEqual(instance.slug, 'pipo-22-eu')

        # test datetimes
        self.assertLess(timezone.now() - instance.created, timedelta(seconds=5))
        almost_zero = instance.updated - instance.created
        self.assertLess(timedelta(0), almost_zero)
        self.assertLess(almost_zero, timedelta(seconds=1))
        instance.save()
        self.assertLess(almost_zero, instance.updated - instance.created)

        # test links
        self.assertEqual(instance.get_absolute_url(), f'/test/{instance.slug}')
        self.assertEqual(str(instance.get_link()), f'<a href="/test/{instance.slug}">{instance}</a>')
        self.assertEqual(instance.get_full_url(), f'https://example.com/test/{instance.slug}')
        self.assertEqual(instance.get_admin_url(), '/admin/testapp/testmodel/1/change/')

        # test query_sum
        self.assertEqual(query_sum(TestModel.objects.all(), 'tests'), 42)

        # test enum_to_choices
        self.assertEqual(instance.get_year_in_school_display(), 'freshman')

        # test ordering
        first = TestModel.objects.create(name='Abc', moment=timezone.now())
        self.assertFalse(TestModel.objects.all().ordered)
        self.assertTrue(TestModel.objects.name_ordered().ordered)
        self.assertEqual(TestModel.objects.name_ordered().first(), first)

    def test_templatetags(self):
        """Test ndh.templatetags."""
        r = self.client.get(reverse('test'))
        self.assertEqual(r.status_code, 200)
        mail = '<span class="mail">test<span class="at"></span>example<span class="dot"></span>org</span>'
        self.assertIn(mail, r.content.decode())

        User.objects.create_user(username='test', password='test')
        self.client.login(username='test', password='test')

        r = self.client.get(reverse('test'))
        self.assertEqual(r.status_code, 200)
        mail = '<span class="mail"><a href="mailto:test@example.org">test@example.org</a></span>'
        self.assertIn(mail, r.content.decode())

        User.objects.create_user(username='super', password='super', is_superuser=True)
        self.client.login(username='super', password='super')

        TestModel.objects.create(name='Pipo 22 é@ü', moment=timezone.now())

        r = self.client.get(reverse('testapp:testmodels'))
        self.assertIn('/admin/testapp/testmodel/', r.content.decode())

        r = self.client.get(TestModel.objects.first().get_absolute_url())
        self.assertIn('/admin/testapp/testmodel/1/change', r.content.decode())

        # test navbar_item
        r = self.client.get(reverse('testapp:testmodels'))
        self.assertIn(
            '\n'.join([
                '<li class="nav-item active"><a class="nav-link" href="/test/">List</a></li>',
                '<li class="nav-item "><a class="nav-link" href="/test/create">Create</a></li>',
            ]), r.content.decode())
        r = self.client.get(reverse('testapp:testmodel-add'))
        self.assertIn(
            '\n'.join([
                '<li class="nav-item "><a class="nav-link" href="/test/">List</a></li>',
                '<li class="nav-item active"><a class="nav-link" href="/test/create">Create</a></li>',
            ]), r.content.decode())

    def test_views(self):
        """Test testapp.views."""
        self.assertEqual(TestModel.objects.count(), 0)
        r = self.client.get(reverse('testapp:testmodel-add'))
        self.assertEqual(r.status_code, 200)
        data = {'name': 'Pipè', 'year_in_school': 2, 'tests': 3, 'moment_0': '2017-12-06', 'moment_1': '03:19:45'}
        r = self.client.post(reverse('testapp:testmodel-add'), data)

        self.assertIn(r.content.decode(), 'Create Test')
        self.assertEqual(TestModel.objects.count(), 1)

        r = self.client.get(reverse('testapp:testmodel-del', args=[TestModel.objects.first().slug]))
        self.assertEqual(r.status_code, 302)

        User.objects.create_user(username='super', password='super', is_superuser=True)
        self.client.login(username='super', password='super')

        r = self.client.get(reverse('testapp:testmodel-del', args=[TestModel.objects.first().slug]))
        self.assertEqual(r.status_code, 200)

        self.assertEqual(TestModel.objects.count(), 1)

        r = self.client.post(reverse('testapp:testmodel-del', args=[TestModel.objects.first().slug]))
        self.assertEqual(r.status_code, 302)

        self.assertEqual(TestModel.objects.count(), 0)

    def test_absolute_url_list(self):
        """Test ndh.models's get_absolute_url for lists."""
        instance = TestModelList.objects.create(name='Pipo 22 é@ü', moment=timezone.now())
        self.assertEqual(instance.get_absolute_url(), reverse('testapp:testmodellists'))

    def test_absolute_url_pk(self):
        """Test ndh.models's get_absolute_url for objects."""
        instance = TestModelPK.objects.create()
        self.assertEqual(instance.get_absolute_url(), f'/test/pk/{instance.pk}')

    def test_utils(self):
        """Test ndh.utils."""
        key, val, no_key, env_file = 'DJANGO_TEST_GET_ENV', 'it=works', 'KEY_WITHOUT_VAL', '.env'
        get_env(env_file)
        if key in os.environ:
            val = os.environ[key]
        else:
            with open(env_file, 'a') as f:
                print(f'{key}={val}', file=f)
        with open(env_file, 'a') as f:
            print(no_key, file=f)
        get_env(env_file)
        self.assertIn(key, os.environ)
        self.assertEqual(os.environ[key], val)
        self.assertNotIn(no_key, os.environ)

        # Clean the file
        with open(env_file) as f:
            lines = f.readlines()
        with open(env_file, 'w') as f:
            f.write('\n'.join(line for line in lines if not line.startswith(key) and not line.startswith(no_key)))

    def test_context_processors(self):
        """Test ndh.context_processors."""
        self.assertNotIn('UTC', self.client.get(reverse('testapp:settings')).content.decode())
        with self.settings(NDH_TEMPLATES_SETTINGS=['TIME_ZONE']):
            self.assertIn('UTC', self.client.get(reverse('testapp:settings')).content.decode())



# Generated by Django 2.0.1 on 2018-02-08 16:01

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('testapp', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TestModelList',
            fields=[
                ('testmodel_ptr',
                 models.OneToOneField(auto_created=True,
                                      on_delete=django.db.models.deletion.CASCADE,
                                      parent_link=True,
                                      primary_key=True,
                                      serialize=False,
                                      to='testapp.TestModel')),
            ],
            options={
                'abstract': False,
            },
            bases=('testapp.testmodel', ),
        ),
    ]



from typing import Dict

from django.conf import settings
from django.http import HttpRequest


def settings_constants(request: HttpRequest) -> Dict[str, str]:
    if hasattr(settings, 'NDH_TEMPLATES_SETTINGS'):
        return {key: getattr(settings, key) for key in settings.NDH_TEMPLATES_SETTINGS}
    return {}



"""Models for the testapp."""
from enum import IntEnum

from django.db import models

from ndh.models import Links, NamedModel, TimeStampedModel
from ndh.utils import enum_to_choices


class TestModel(Links, TimeStampedModel, NamedModel):
    """Main test model."""
    YEARS_IN_SCHOOL = IntEnum('years_in_school', 'freshman sophomore junior senior')

    year_in_school = models.IntegerField(choices=enum_to_choices(YEARS_IN_SCHOOL), default=1)
    tests = models.IntegerField(default=42)
    moment = models.DateTimeField()


class TestModelList(TestModel):
    """Test model for get__absolute_url for lists."""
    absolute_url_detail = False


class TestModelPK(Links, TimeStampedModel):
    """Test model for get_absolute_url for objects."""
    pass
